<?php
// Prevent direct access to this file
if (!defined('ABSPATH')) {
    exit;
}

require_once ABSPATH . 'wp-admin/includes/plugin.php';
require_once ABSPATH . 'wp-admin/includes/theme.php';

class GPL_Updater {
    public function __construct() {
        add_filter("site_transient_update_plugins", [$this, "setTransientGPL"], 99999999);
        add_filter('pre_set_site_transient_update_plugins', [$this, "setTransientGPL"], 99999999);
        add_filter("site_transient_update_themes", [$this, "setTransientGPL"], 99999999);
        add_filter('pre_set_site_transient_update_themes', [$this, "setTransientGPL"], 99999999);
        add_filter('upgrader_pre_download', '__return_false', 99999999);

        add_action('wp', [$this, 'gpl_schedule_update_check']);
        add_action('gpl_check_updates_hook', [$this, 'gpl_check_and_store_updates']);
        add_action('activated_plugin', [$this, 'gpl_check_and_store_updates']);
        add_action('switch_theme', [$this, 'gpl_check_and_store_updates']);
        add_action('current_screen', [$this, 'gpl_maybe_update_transients']);
        add_action('upgrader_process_complete', [$this, 'clear_update_cache'], 10, 2);
        add_action('admin_init', [$this, 'remove_update_notifications']);

        add_filter('cron_schedules', [$this, 'gpl_add_cron_interval']);
    }

    public function remove_update_notifications() {
        // Remove update notifications if items are already updated
        $stored_updates = $this->gpl_get_stored_updates();
        $need_update = false;

        if (!empty($stored_updates['plugins'])) {
            foreach ($stored_updates['plugins'] as $plugin_file => $plugin_info) {
                $plugin_data = get_plugin_data(WP_PLUGIN_DIR . '/' . $plugin_file);
                if (version_compare($plugin_data['Version'], $plugin_info['new_version'], '>=')) {
                    unset($stored_updates['plugins'][$plugin_file]);
                } else {
                    $need_update = true;
                }
            }
        }

        if (!empty($stored_updates['themes'])) {
            foreach ($stored_updates['themes'] as $theme_slug => $theme_info) {
                $theme = wp_get_theme($theme_slug);
                if (version_compare($theme->get('Version'), $theme_info['new_version'], '>=')) {
                    unset($stored_updates['themes'][$theme_slug]);
                } else {
                    $need_update = true;
                }
            }
        }

        update_option('gpl_available_updates', $stored_updates);

        if (!$need_update) {
            delete_site_transient('update_plugins');
            delete_site_transient('update_themes');
            wp_clean_plugins_cache(true);
            wp_clean_themes_cache(true);
        }
    }

    public function setTransientGPL($transient) {
        if (empty($transient->checked)) {
            return $transient;
        }

        $update_data = $this->gpl_get_stored_updates();
        $transient->response = isset($transient->response) ? $transient->response : array();

        if (current_filter() === 'site_transient_update_plugins' || current_filter() === 'pre_set_site_transient_update_plugins') {
            if (!empty($update_data['plugins'])) {
                foreach ($update_data['plugins'] as $plugin_file => $plugin_info) {
                    if (!file_exists(WP_PLUGIN_DIR . '/' . $plugin_file)) {
                        continue;
                    }
                    $plugin_data = get_plugin_data(WP_PLUGIN_DIR . '/' . $plugin_file);
                    if (version_compare($plugin_data['Version'], $plugin_info['new_version'], '<')) {
                        $transient->response[$plugin_file] = (object) array(
                            'slug' => $plugin_info['slug'],
                            'plugin' => $plugin_file,
                            'new_version' => $plugin_info['new_version'],
                            'url' => $plugin_info['url'],
                            'package' => isset($plugin_info['package']) ? $plugin_info['package'] : '',
                        );
                    } else {
                        unset($transient->response[$plugin_file]);
                    }
                }
            }
        } elseif (current_filter() === 'site_transient_update_themes' || current_filter() === 'pre_set_site_transient_update_themes') {
            if (!empty($update_data['themes'])) {
                foreach ($update_data['themes'] as $theme_slug => $theme_info) {
                    $theme = wp_get_theme($theme_slug);
                    if ($theme->exists() && version_compare($theme->get('Version'), $theme_info['new_version'], '<')) {
                        $transient->response[$theme_slug] = array(
                            'theme' => $theme_slug,
                            'new_version' => $theme_info['new_version'],
                            'url' => $theme_info['url'],
                            'package' => isset($theme_info['package']) ? $theme_info['package'] : '',
                        );
                    } else {
                        unset($transient->response[$theme_slug]);
                    }
                }
            }
        }

        return $transient;
    }

    public function gpl_check_and_store_updates() {
        $updates = array(
            'plugins' => array(),
            'themes' => array()
        );

        // Fetch the JSON data
        $response = wp_remote_get('https://www.xyztheme.com/gplcheck.json');

        if (is_wp_error($response)) {
            error_log('Error fetching GPL data: ' . $response->get_error_message());
            return $updates;
        }

        $body = wp_remote_retrieve_body($response);
        $gpl_data = json_decode($body, true);

        if (!$gpl_data) {
            error_log('Invalid JSON data retrieved from GPL site.');
            return $updates;
        }

        // Check plugins
        $installed_plugins = get_plugins();
        $plugins_to_update = array();

        foreach ($installed_plugins as $plugin_path => $plugin_data) {
            if (isset($gpl_data[$plugin_path])) {
                $gpl_info = $gpl_data[$plugin_path];
                if (version_compare($plugin_data['Version'], $gpl_info['version'], '<')) {
                    $plugins_to_update[] = $plugin_path;
                    $updates['plugins'][$plugin_path] = array(
                        'current_version' => $plugin_data['Version'],
                        'new_version' => $gpl_info['version'],
                        'slug' => $gpl_info['slug'],
                        'plugin' => $plugin_path,
                        'name' => $gpl_info['name'],
                        'author' => $gpl_info['authorname'],
                        'url' => $gpl_info['pluginuri'],
                        'last_update' => $gpl_info['last_update'],
                    );
                }
            }
        }

        // Check themes
        $installed_themes = wp_get_themes();
        $themes_to_update = array();

        foreach ($installed_themes as $theme_slug => $theme) {
            if (isset($gpl_data[$theme_slug])) {
                $gpl_info = $gpl_data[$theme_slug];
                if (version_compare($theme->get('Version'), $gpl_info['version'], '<')) {
                    $themes_to_update[] = $theme_slug;
                    $updates['themes'][$theme_slug] = array(
                        'current_version' => $theme->get('Version'),
                        'new_version' => $gpl_info['version'],
                        'slug' => $theme_slug,
                        'theme' => $theme_slug,
                        'name' => $gpl_info['name'],
                        'author' => $gpl_info['authorname'],
                        'url' => $gpl_info['pluginuri'],
                        'last_update' => $gpl_info['last_update'],
                    );
                }
            }
        }

        $items_to_update = array_merge($plugins_to_update, $themes_to_update);

        if (!empty($items_to_update)) {
            // Get package details
            $package_details = $this->gpl_get_package_details($items_to_update);

            // Merge package details with update information
            foreach ($package_details as $item_path => $details) {
                if (isset($updates['plugins'][$item_path])) {
                    $updates['plugins'][$item_path]['package'] = $details['package'];
                } elseif (isset($updates['themes'][$item_path])) {
                    $updates['themes'][$item_path]['package'] = $details['package'];
                }
            }
        }

        // Store the updates in the database
        update_option('gpl_available_updates', $updates);

        return $updates;
    }

    public function gpl_get_package_details($items) {
    if (empty($items)) {
        return array();
    }

    // Get the saved token
    $token = get_option('gpl_auth_token');

    // Check if the token is valid
    $expiry = get_option('gpl_auth_token_expiry');
    if (!$token || !$expiry || strtotime($expiry) <= time()) {
        error_log('Invalid or expired token.');
        return array();
    }

    $response = wp_remote_post(GPL_AUTH_PACKAGE_DETAILS, array(
        'body' => json_encode($items),
        'headers' => array(
            'Content-Type' => 'application/json',
            'Authorization' => 'Bearer ' . $token
        )
    ));

    if (is_wp_error($response)) {
        error_log('Error fetching package details: ' . $response->get_error_message());
        return array();
    }

    $body = wp_remote_retrieve_body($response);
    $package_details = json_decode($body, true);

    if (!$package_details) {
        error_log('Invalid JSON data retrieved for package details.');
        return array();
    }

    return $package_details;
}

    public function gpl_get_stored_updates() {
        return get_option('gpl_available_updates', array('plugins' => array(), 'themes' => array()));
    }

    public function gpl_schedule_update_check() {
        if (!wp_next_scheduled('gpl_check_updates_hook')) {
            wp_schedule_event(time(), 'thirty_minutes', 'gpl_check_updates_hook');
        }
    }

    public function gpl_maybe_update_transients() {
        $screen = get_current_screen();
        if ($screen && in_array($screen->id, array('plugins', 'themes', 'update-core'))) {
            $last_check = get_option('gpl_last_admin_check', 0);
            $current_time = time();
            if ($current_time - $last_check > 10800) { // 3 hours = 10800 seconds
                $this->gpl_check_and_store_updates();
                update_option('gpl_last_admin_check', $current_time);
            }
        }
    }

    public function gpl_add_cron_interval($schedules) {
        $schedules['thirty_minutes'] = array(
            'interval' => 1800,
            'display'  => esc_html__('Every 30 Minutes'),
        );
        return $schedules;
    }

    public function gpl_manual_check_updates() {
        $this->gpl_check_and_store_updates();
    }

    public function clear_update_cache($upgrader, $options) {
        if (!isset($options['action']) || !isset($options['type'])) {
            return;
        }

        if ($options['action'] !== 'update') {
            return;
        }

        // Clear all caches
        delete_site_transient('update_plugins');
        delete_site_transient('update_themes');
        wp_clean_plugins_cache(true);
        wp_clean_themes_cache(true);
        
        $stored_updates = get_option('gpl_available_updates', array());

        if ($options['type'] === 'plugin') {
            if (isset($options['plugins']) && is_array($options['plugins'])) {
                foreach ($options['plugins'] as $plugin_file) {
                    unset($stored_updates['plugins'][$plugin_file]);
                }
            }
        } elseif ($options['type'] === 'theme') {
            if (isset($options['themes']) && is_array($options['themes'])) {
                foreach ($options['themes'] as $theme_slug) {
                    unset($stored_updates['themes'][$theme_slug]);
                }
            }
        }

        update_option('gpl_available_updates', $stored_updates);
        
        // Force immediate refresh of update data
        wp_schedule_single_event(time(), 'gpl_check_updates_hook');
    }
}

$gpl_updater = new GPL_Updater();

// Activation hook
register_activation_hook(__FILE__, function() use ($gpl_updater) {
    $gpl_updater->gpl_check_and_store_updates();
});
